from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from collective.lineage.interfaces import IChildSite
from lineage.registry.proxy import LineageRecords
from lineage.registry.proxy import REGISTRY_NAME
from lineage.registry.proxy import _LineageRecords
from zope.component.hooks import getSite
import logging

log = logging.getLogger(__name__)


def upgrade_from_persistent(context):
    portal = getSite()
    catalog = getToolByName(portal, 'portal_catalog')
    query = {}
    query['object_provides'] = IChildSite.__identifier__
    results = catalog(**query)
    log.info('There are {0} in total, stating migration...'.format(
        len(results)))

    for result in sorted(results, key=lambda it: it.getURL()):
        obj = result.getObject()
        reg = getattr(obj, REGISTRY_NAME, None)
        if reg:
            log.info(
                'migration for {0}'.format('/'.join(reg.getPhysicalPath()))
            )
            reg = aq_base(reg)
            if not getattr(reg, '__parent__', False):
                # set the parent persitently
                reg.__parent__ = obj
                log.info('set registry parent')

            if not getattr(reg._records, '__parent__', False):
                reg._records.__parent__ = reg
                log.info('set records parent')

            if isinstance(reg._records, _LineageRecords):
                oldrecs = reg._records
                reg._records = LineageRecords(reg)

                for key, val in oldrecs.items():
                    try:
                        reg._records[key] = val
                    except KeyError:
                        # I had a case, where the record with the key
                        # ``plone.site_logo`` raised a KeyError, when
                        # accessing val.value.
                        # See: plone.registry.record.Record._get_value
                        # When removing the parent, the value was accessed
                        # directly via val._value and all was O.K.
                        del val.__parent__
                        reg._records[key] = val
                    except:
                        log.warn('could migrate {0} for {1}'.format(key, reg))
                log.info('migrate records')

        else:
            log.warn('could not migrate {0}'.format(reg.getPhysicalPath()))



# -*- coding: utf-8 -*-
from collective.lineage.interfaces import IChildSite
from lineage.registry.proxy import REGISTRY_NAME
from lineage.registry.utils import disableRegistry
from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            'lineage.registry:uninstall',
        ]


def uninstall(context):
    """Uninstall script"""
    # query all childsites from catalog
    for brain in api.content.find(object_provides=IChildSite):
        obj = brain.getObject()
        if REGISTRY_NAME in obj:
            disableRegistry(obj)
            obj.manage_delObjects([REGISTRY_NAME])



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '1.5'
short_description = u"Lineage Add-On: Child Site Local Plone Registry"
long_description = u'\n\n'.join([
    open('README.rst').read(),
    open('CHANGES.rst').read(),
    open('LICENSE.rst').read(),
])


setup(
    name='lineage.registry',
    version=version,
    description=short_description,
    long_description=long_description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.0',
        'Framework :: Plone :: 5.1',
        'Framework :: Plone :: 5.2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    keywords='plone lineage subsite',
    author='BlueDynamics Alliance',
    author_email='dev@bluedynamics.com',
    url=u'https://github.com/collective/lineage.registry',
    license='GNU General Public Licence',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['lineage'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'collective.lineage',
    ],
    extras_require={
        'test': [
            'collective.lineage [test]',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
